@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
+import multiprocessing
 import os
-import threading
-import datetime
-import time
 from functools import lru_cache, wraps
+from multiprocessing import shared_memory
+import threading
 
 import math
-import numpy
-import pandas
+import numpy as np
+import pandas as pd
 
-from .utils import *
 from .technical_analysis import *
+from .utils import *
 
 
 def assert_quote_init(func):
@@ -25,19 +25,56 @@ def assert_quote_init(func):
     return _wrapper
 
 
+def _rehabilitation_worker(start=None, end=None, fq_date=None, fq_vol=False, xrxd=None, precision=2
+                           , data=None, shm_name=None, shape=None):
+    if not xrxd:
+        return
+
+    if shm_name:
+        shm = shared_memory.SharedMemory(name=shm_name)
+        data = np.ndarray(shape, dtype=np.float64, buffer=shm.buf)
+
+    if not start:
+        start = 0
+    if not end:
+        end = len(data)
+    if not fq_date:
+        fq_date = pd.Timestamp(time.localtime('%Y-%m-%d'))
+
+    values = data[start:end]
+    n = data.shape[1] - 1 - (1 if fq_vol else 0)
+
+    for row in xrxd:
+        ts_date = row['date']
+        r = 10 / (10 + (row['dividend_ratio'] if not math.isnan(row['dividend_ratio']) else 0) + (
+            row['transfer_ratio'] if not math.isnan(row['transfer_ratio']) else 0))
+        v = (row['bonus_ratio'] if not math.isnan(row['bonus_ratio']) else 0) / 10
+
+        # 前复权处理
+        if ts_date <= fq_date:
+            #   c = data[data['datetime'] < ts_date]
+            c = np.where(values[:, 0] < ts_date.value)
+            values[c, 1: 1 + n] = np.round((values[c, 1: 1 + n] - v) * r, precision)
+            if fq_vol:
+                values[c, -1] = np.round(values[c, -1] / r)
+        # 后复权处理
+        else:
+            #    c = data[data['datetime'] >= ts_date]
+            c = np.where(values[:, 0] >= ts_date.value)
+            values[c, 1: 1 + n] = np.round(values[c, 1: 1 + n] / r + v, precision)
+            if fq_vol:
+                values[c, -1] = np.round(values[c, -1] * r)
+
+    if shm_name:
+        shm.close()
+
+
+def _arithmetic_worker(df, func, kw):
+    arithmetic_reset()
+    return df.apply(func, **kw)
+
+
 class QuoteData(object):
-    '''
-    _instance_lock = threading.Lock()
-
-    def __new__(cls, *args, **kwargs):
-        """单例模式"""
-        if not hasattr(cls, "_instance"):
-            with cls._instance_lock:
-                if not hasattr(cls, "_instance"):
-                    cls._instance = object.__new__(cls)
-        return cls._instance
-    '''
-
     _threading_local = threading.local()
 
     def __new__(cls, *args, **kwargs):
@@ -57,6 +94,12 @@ class QuoteData(object):
             self._data_path = ''
         if data_path:
             self.init(data_path)
+        if not hasattr(self, '_options'):
+            self._options = {
+                'debug': False,
+                'refer_num': 200,
+                'multi_process': False
+            }
 
     @assert_quote_init
     @lru_cache(maxsize=None, typed=True)
@@ -89,7 +132,7 @@ class QuoteData(object):
                 parse_dates = [date_columns]
             else:
                 raise ValueError('date_columns is error.')
-            result = pandas.read_csv(data_file, dtype={'code': str}, parse_dates=parse_dates)
+            result = pd.read_csv(data_file, dtype={'code': str}, parse_dates=parse_dates)
         elif extname == 'txt':
             with open(data_file, 'r') as f:
                 result = f.readlines()
@@ -114,6 +157,31 @@ class QuoteData(object):
         """
         return self._data_path
 
+    def get_option(self, name):
+        """获取可选项值
+
+        参数:
+            name: 可选项名
+
+        返回值:
+            可选项值
+
+        """
+        if name not in self._options:
+            raise ValueError('option %s is not exists.' % name)
+
+    def set_option(self, name, value):
+        """设置可选项值
+
+        参数:
+            name: 可选项名
+            value: 可选项值
+
+        """
+        if name not in self._options:
+            raise ValueError('option %s is not exists.' % name)
+        self._options[name] = value
+
     @classmethod
     def _get_price_precision(cls, security) -> int:
         """获取价格精度
@@ -222,8 +290,13 @@ class QuoteData(object):
             start = trade_list[0].timetuple()
         return start
 
+    def trace(self, *args):
+        if self._options.get('debug', False):
+            print(time.strftime('%Y-%m-%d %H:%M:%S'), *args)
+
     @assert_quote_init
-    def execute_rehabilitation(self, data, fq='pre', fq_vol=False, columns=None) -> pandas.DataFrame:
+    def execute_rehabilitation(self, data, fq='pre', fq_vol=False, start_date=None, end_date=None
+                               , columns=None, inplace=False, skip_sort=False) -> pd.DataFrame:
         """执行复权操作
 
         复权逻辑：
@@ -232,74 +305,209 @@ class QuoteData(object):
             当 fq == 'post' 时，复权参考日为 start_date
             当 除权除息日 <= 复权参考日 时，所有 < 除权除息日 的数据进行前复权
             当 除权除息日 > 复权参考日 时，所有 >= 除权除息日 的数据进行后复权
+            start_date 与 end_date 指除权除息发生时间，仅复权该时间段内的行情数据
 
         参数:
             data: 输入数据
             fq: 复权方式或复权参考日
             fq_vol: 是否复权成交量：默认值 False
+            start_date: 复权开始日，默认值为 data.datetime 的最小值
+            end_date: 复权结束日，默认值为 data.datetime 的最大值
             columns: 需要复权的字段列表
+            inplace: 是否复盖输入数据，默认值 False
+            skip_sort: 跳过排序，默认值 False；传入的 data 必须符合 code, datetime 排序，否则处理结果会有误
 
         返回值: 输出数据
 
         """
+        proc_start = time.time()
         default_cols_px = ['open', 'close', 'high', 'low', 'pre_close', 'high_limit', 'low_limit']
         default_cols_vm = ['volume']
 
-        df = data
+        df = data if inplace else data.copy()
         if not fq or len(df) == 0:
             return df
         if not columns:
             columns = df.columns
 
         security = list(set(df['code'].tolist()))
-        start_date = min(df['datetime']).strftime('%Y-%m-%d')
-        end_date = max(df['datetime']).strftime('%Y-%m-%d')
+        start_date = formatter_st(start_date, min(df['datetime']).strftime('%Y-%m-%d'))
+        end_date = formatter_st(end_date, max(df['datetime']).strftime('%Y-%m-%d'))
+        self.trace('rehabilitate from %s to %s'
+                   % (time.strftime('%Y-%m-%d', start_date), time.strftime('%Y-%m-%d', end_date)))
 
         if fq == 'pre':
-            ts_ref = pandas.Timestamp(time.strftime('%Y-%m-%d', formatter_st(end_date)))
+            fq_date = pd.Timestamp(time.strftime('%Y-%m-%d', end_date))
         elif fq == 'post':
-            ts_ref = pandas.Timestamp(time.strftime('%Y-%m-%d', formatter_st(start_date)))
+            fq_date = pd.Timestamp(time.strftime('%Y-%m-%d', start_date))
         else:
-            ts_ref = pandas.Timestamp(time.strftime('%Y-%m-%d', formatter_st(fq)))
+            fq_date = pd.Timestamp(time.strftime('%Y-%m-%d', formatter_st(fq)))
 
         xrxd = self.get_security_xrxd(security=security, start_date=start_date, end_date=end_date)
         if xrxd is None:
             return df
+        xrxd.sort_values(by=['code', 'date'], inplace=True)
+        xrxd.set_index('code', drop=True, inplace=True)
 
-        fq_cols_px = [x for x in list(df.columns) if x in columns and x in default_cols_px]
-        fq_cols_vm = [x for x in list(df.columns) if x in columns and x in default_cols_vm] if fq_vol else []
-        if not fq_cols_px and not fq_cols_vm:
+        cols_px = [x for x in list(df.columns) if x in columns and x in default_cols_px]
+        cols_vm = [x for x in list(df.columns) if x in columns and x in default_cols_vm] if fq_vol else []
+        if not cols_px and not cols_vm:
             return df
-        for index, row in xrxd.iterrows():
-            ts_date = pandas.Timestamp(row['date'])
+        self.trace('exec', time.time() - proc_start)
 
-            # print(row['code'], row['date'], r, v)
-            r = 10 / (10 + (row['dividend_ratio'] if not math.isnan(row['dividend_ratio']) else 0) + (
-                row['transfer_ratio'] if not math.isnan(row['transfer_ratio']) else 0))
-            v = (row['bonus_ratio'] if not math.isnan(row['bonus_ratio']) else 0) / 10
-            if fq_cols_px:
-                n = self._get_price_precision(row['code'])
-                if ts_date <= ts_ref:
-                    c = df.loc[(df['code'] == row['code']) & (df['datetime'] < ts_date), fq_cols_px]
-                    if len(c) > 0:
-                        c = round((c - v) * r, n)
-                        df.loc[(df['code'] == row['code']) & (df['datetime'] < ts_date), fq_cols_px] = c
-                else:
-                    c = df.loc[(df['code'] == row['code']) & (df['datetime'] >= ts_date), fq_cols_px]
-                    if len(c) > 0:
-                        c = round(c / r + v, n)
-                        df.loc[(df['code'] == row['code']) & (df['datetime'] >= ts_date), fq_cols_px] = c
-            if fq_cols_vm:
-                if ts_date <= ts_ref:
-                    c = df.loc[(df['code'] == row['code']) & (df['datetime'] < ts_date), fq_cols_vm]
-                    if len(c) > 0:
-                        c = round(c / r)
-                        df.loc[(df['code'] == row['code']) & (df['datetime'] < ts_date), fq_cols_vm] = c
-                else:
-                    c = df.loc[(df['code'] == row['code']) & (df['datetime'] >= ts_date), fq_cols_vm]
-                    if len(c) > 0:
-                        c = round(c * r)
-                        df.loc[(df['code'] == row['code']) & (df['datetime'] >= ts_date), fq_cols_vm] = c
+        if not skip_sort:
+            df.sort_values(by=['code', 'datetime'], axis=0, ascending=True, inplace=True)
+            df.reset_index(drop=True, inplace=True)
+
+        shape = (len(df), 1 + len(cols_px) + len(cols_vm))
+        multi_process = self._options.get('multi_process', False)
+        if multi_process:
+            self.trace('rehabilitation within multi-process')
+
+            shm = shared_memory.SharedMemory(create=True, size=shape[0] * shape[1] * 8)
+            np_data = np.ndarray(shape=shape, dtype=np.float64, buffer=shm.buf)
+            np_data[:, 0] = df['datetime'].values.astype('float')
+            np_data[:, 1:] = df[cols_px + cols_vm].values
+            self.trace('exec', time.time() - proc_start)
+
+            operate_params = list()
+            for code, group in df.groupby(by='code'):
+                if len(group) == 0 or code not in xrxd.index:
+                    continue
+                kw = {'start': group.index[0], 'end': group.index[-1] + 1, 'fq_date': fq_date, 'fq_vol': fq_vol
+                    , 'xrxd': xrxd.loc[[code]].to_dict(orient='records'), 'precision': self._get_price_precision(code)
+                    , 'shm_name': shm.name, 'shape': shape}
+                operate_params.append(kw)
+
+            tasks = []
+            pool = multiprocessing.Pool(processes=multi_process if type(multi_process) is int else None)
+            for kw in operate_params:
+                tasks.append(pool.apply_async(_rehabilitation_worker, kwds=kw))
+            pool.close()
+            pool.join()
+
+            df[cols_px + cols_vm] = np_data[:, 1:]
+            shm.close()
+            shm.unlink()
+        else:
+            np_data = np.empty(shape=shape, dtype=np.float64)
+            np_data[:, 0] = df['datetime'].values.astype('float')
+            np_data[:, 1:] = df[cols_px + cols_vm].values
+            for code, group in df.groupby(by='code'):
+                if len(group) == 0 or code not in xrxd.index:
+                    continue
+                # self.trace('rehabilitate %s' % code)
+                kw = {'start': group.index[0], 'end': group.index[-1] + 1, 'fq_date': fq_date, 'fq_vol': fq_vol
+                    , 'xrxd': xrxd.loc[[code]].to_dict(orient='records'), 'precision': self._get_price_precision(code)
+                    , 'data': np_data}
+                _rehabilitation_worker(**kw)
+            self.trace('exec', time.time() - proc_start)
+            df[cols_px + cols_vm] = np_data[:, 1:]
+            self.trace('exec', time.time() - proc_start)
+
+        self.trace('exec', time.time() - proc_start)
+        return df
+
+    @assert_quote_init
+    def execute_technical_analysis(self, data, indications, inplace=False, skip_sort=False) -> pd.DataFrame:
+        """执行技术指标运算
+
+        参数:
+            data: 输入数据
+            indications: 技术指标及参与，单个或List
+                         单个指标：（'MA', 5)，
+                         多个指标：[('MA', 5), ('KDJ', 9, 3, 3)]
+                         可省略参数：['MA', 'KDJ']
+            inplace: 是否复盖输入数据，默认值 False
+            skip_sort: 跳过排序，默认值 False；传入的 data 必须符合 code, datetime 排序，否则处理结果会有误
+
+        返回值: 输出数据
+
+        """
+        proc_start = time.time()
+        df = data if inplace else data.copy()
+
+        if not skip_sort:
+            df.sort_values(by=['code', 'datetime'], axis=0, ascending=True, inplace=True)
+            df.reset_index(drop=True, inplace=True)
+
+        operate_params = []
+        cols_techansi = []
+        indications = formatter_list(indications)
+        for v in indications:
+            v = [v] if type(v) is str else list(v)
+            c = v.pop(0)
+            func = globals().get(c)
+            if not func:
+                print('%s is not exists.' % c)
+                continue
+            if not hasattr(func, '__wrapped__'):
+                print('%s is unavailable.' % c)
+                continue
+            f_code = getattr(func, '__wrapped__').__code__
+            f_args = [x for x in f_code.co_varnames[:f_code.co_argcount]]
+            if len(f_args) > 0 and f_args[0] == 'X' and len(v) > 0 and type(v[0]) is not str:
+                f_args.pop(0)
+            kw = dict(axis=1)
+            for i in range(len(v)):
+                if i >= len(f_args):
+                    break
+                kw[f_args[i]] = v[i]
+
+            # 获取返回值字段名
+            cols_fn = arithmetic_returns(f_code)
+            if not cols_fn:
+                cols_fn = c
+            if type(cols_fn) is str:
+                cols_fn = [cols_fn]
+            if not cols_fn:
+                raise Exception('columns name not support.')
+
+            # 字段重名处理
+            if [x for x in cols_fn if x in cols_techansi]:
+                c_arr = [str(x) for x in v]
+                if len(cols_fn) > 1:
+                    c_arr.insert(0, c)
+                    cols_fn = ['_'.join(c_arr) + '_' + x for x in cols_fn]
+                elif len(cols_fn) == 1:
+                    if cols_fn[0] == c:
+                        c_arr.insert(0, c)
+                        cols_fn = ['_'.join(c_arr)]
+                    else:
+                        c_arr.insert(0, cols_fn[0])
+                        cols_fn = ['_'.join(c_arr)]
+
+            # 加入操作参数
+            operate_params.append((cols_fn, func, kw))
+            cols_techansi.extend(cols_fn)
+
+        # 提交指标函数处理
+        multi_process = self._options.get('multi_process', False)
+        if multi_process:
+            self.trace('arithmetic within multi-process')
+
+            tasks = []
+            pool = multiprocessing.Pool(processes=multi_process if type(multi_process) is int else None)
+            for code, group in df.groupby(by='code'):
+                for cols_fn, func, kw in operate_params:
+                    tasks.append((cols_fn, pool.apply_async(_arithmetic_worker, (group, func, kw))))
+            pool.close()
+            pool.join()
+
+            df[cols_techansi] = np.nan
+            for cols_fn, result in tasks:
+                se = result.get()
+                se.dropna(inplace=True)
+                if not se.empty:
+                    df.loc[se.index, cols_fn] = se.tolist() if len(cols_fn) > 1 else se
+        else:
+            for cols_fn, func, kw in operate_params:
+                se = _arithmetic_worker(df, func, kw)
+                se.dropna(inplace=True)
+                if not se.empty:
+                    df[cols_fn] = pd.DataFrame(se.tolist(), index=se.index) if len(cols_fn) > 1 else se
+
+        # self.trace('exec', time.time() - proc_start)
         return df
 
     @assert_quote_init
@@ -328,7 +536,7 @@ class QuoteData(object):
             if not k or not v:
                 continue
             result[k] = v
-        # print(result)
+        # self.trace(result)
         stime = result.get('last_date')
         if not stime:
             raise Exception('the quote_data not found.')
@@ -354,9 +562,9 @@ class QuoteData(object):
 
         start_date = formatter_st(start_date) if start_date else None
         end_date = formatter_st(end_date) if end_date else None
-        dt_start = numpy.datetime64(
+        dt_start = np.datetime64(
             datetime.datetime.fromtimestamp(time.mktime(start_date))) if start_date else None
-        dt_end = numpy.datetime64(datetime.datetime.fromtimestamp(time.mktime(end_date))) if end_date else None
+        dt_end = np.datetime64(datetime.datetime.fromtimestamp(time.mktime(end_date))) if end_date else None
 
         filename = 'quote-tdays.csv'
         df = self._fetch_data(filename, date_columns='date')
@@ -366,18 +574,18 @@ class QuoteData(object):
         result = df['date'].values
         if dt_start or dt_end or count:
             if not dt_end:
-                dt_end = numpy.datetime64(datetime.date.today())
+                dt_end = np.datetime64(datetime.date.today())
             if dt_start:
-                result = result[numpy.where(result >= dt_start)]
+                result = result[np.where(result >= dt_start)]
             if dt_end:
-                result = result[numpy.where(result <= dt_end)]
+                result = result[np.where(result <= dt_end)]
             if count and count > 0:
                 result = result[-count:]
         result = [datetime.date.fromtimestamp(x.astype(datetime.datetime) / 1e9) for x in result]
         return result
 
     @assert_quote_init
-    def get_security_info(self, security=None, fields=None, types=None) -> pandas.DataFrame:
+    def get_security_info(self, security=None, fields=None, types=None) -> pd.DataFrame:
         """获取证券信息数据
 
         参数:
@@ -399,7 +607,7 @@ class QuoteData(object):
         fields = formatter_list(fields)
         types = formatter_list(types, 'stock')
 
-        result = pandas.DataFrame()
+        result = pd.DataFrame()
         filename = 'quote-ctb.csv'
         df = self._fetch_data(filename, date_columns=('start_date', 'end_date'))
         df = df.copy()
@@ -419,7 +627,7 @@ class QuoteData(object):
         return result
 
     @assert_quote_init
-    def get_security_xrxd(self, security, start_date=None, end_date=None, count=None) -> pandas.DataFrame:
+    def get_security_xrxd(self, security, start_date=None, end_date=None, count=None) -> pd.DataFrame:
         """获取证券除权除息数据
 
         参数:
@@ -454,7 +662,7 @@ class QuoteData(object):
         return df
 
     @assert_quote_init
-    def get_quote_static(self, date=None, fields=None, exch=None) -> pandas.DataFrame:
+    def get_quote_static(self, date=None, fields=None, exch=None) -> pd.DataFrame:
         """获取静态行情数据
 
         参数:
@@ -482,7 +690,7 @@ class QuoteData(object):
         fields = formatter_list(fields)
         exch = formatter_list(exch, ['SH', 'SZ'])
         curr_date = formatter_st(date, self.get_quote_date())
-        result = pandas.DataFrame()
+        result = pd.DataFrame()
         for x in exch:
             filename = os.path.join('static', x, time.strftime('%Y', curr_date),
                                     x + '_STATIC_' + time.strftime('%Y%m%d', curr_date) + '.csv')
@@ -491,7 +699,7 @@ class QuoteData(object):
                 continue
             df = df.copy()
             df['code'] = df['code'] + '.' + x
-            df['paused'] = df['paused'].astype('bool_')
+            df['paused'] = df['paused'].astype('int')
             if fields:
                 cols_retain = ['datetime', 'code'] + fields
                 cols_all = list(df.columns)
@@ -569,7 +777,7 @@ class QuoteData(object):
                 evaluate_days = None
             if evaluate_days < len(security):
                 mode_static = True
-        result = pandas.DataFrame()
+        result = pd.DataFrame()
         if mode_static:
             # 从静态文件获取数据
             check_security_count = count and skip_paused
@@ -583,7 +791,7 @@ class QuoteData(object):
                 # df = df.copy()
                 df = df[df['code'].isin(sec_list)]
                 if skip_paused:
-                    df = df[~df['paused']]
+                    df = df[df['paused'] == 0]
                 if check_security_count:
                     for c in df['code'].tolist():
                         if c not in sec_num:
@@ -592,7 +800,7 @@ class QuoteData(object):
                         if sec_num[c] < count:
                             continue
                         sec_list.remove(c)
-                df = self.execute_rehabilitation(df, fq, fq_vol, cols_retain)
+                # self.execute_rehabilitation(df, fq, fq_vol, columns=cols_retain, inplace=True)
                 result = result.append(df, ignore_index=True)
                 if check_security_count:
                     if not sec_list:
@@ -609,9 +817,9 @@ class QuoteData(object):
             result.reset_index(drop=True, inplace=True)
         else:
             # 从K线文件获取数据
-            dt_start = numpy.datetime64(
+            dt_start = np.datetime64(
                 datetime.datetime.fromtimestamp(time.mktime(start_date))) if start_date else None
-            dt_end = numpy.datetime64(datetime.datetime.fromtimestamp(time.mktime(end_date)))
+            dt_end = np.datetime64(datetime.datetime.fromtimestamp(time.mktime(end_date)))
             append_paused = 'paused' in fields or skip_paused
             for c in security:
                 p = c.find('.')
@@ -627,22 +835,23 @@ class QuoteData(object):
                 df = df.copy()
                 if append_paused:
                     if period_type == 'day':
-                        df['paused'] = df.apply(lambda x: x['volume'] == 0 and x['money'] == 0, axis=1)
+                        df['paused'] = df.apply(lambda x: 1 if x['volume'] == 0 and x['money'] == 0 else 0, axis=1)
                     elif period_type == 'mn1':
                         c_period = 'day'
                         c_file = os.path.join(c_period, exc_code, exc_code + '_' + c_period.upper()
                                               + '_' + sec_code + '.csv')
                         c_df = self._fetch_data(c_file, date_columns='datetime')
                         c_skip_days = c_df[(c_df['volume'] == 0) & (c_df['money'] == 0)]['datetime'].tolist()
-                        df['paused'] = df['datetime'].astype('datetime64[D]').isin(c_skip_days)
+                        df['paused'] = df['datetime'].apply(
+                            lambda x: 1 if x.astype('datetime64[D]').isin(c_skip_days) else 0)
                 if skip_paused:
-                    df = df[~df['paused']]
+                    df = df[df['paused'] == 0]
                 if not prep_count:
                     df = df[(df['datetime'] >= dt_start) & (df['datetime'] <= dt_end)]
                 else:
                     df = df[df['datetime'] <= dt_end][-prep_count:]
                 df.insert(loc=0, column='code', value=sec_code + '.' + exc_code)
-                df = self.execute_rehabilitation(df, fq, fq_vol, cols_retain)
+                # self.execute_rehabilitation(df, fq, fq_vol, columns=cols_retain, inplace=True)
                 result = result.append(df, ignore_index=True)
 
         # 多周期分组聚合
@@ -667,7 +876,7 @@ class QuoteData(object):
                         'volume': 'sum', 'money': 'sum', 'paused': 'min'}
             for k in [x for x in agg_dict.keys() if x not in cols_retain]:
                 del agg_dict[k]
-            result = result.groupby(['code', 'period']).agg(agg_dict)
+            result = result.groupby(by=['code', 'period']).agg(agg_dict)
             result.reset_index(inplace=True)
             result.drop(columns='period', inplace=True)
 
@@ -680,6 +889,9 @@ class QuoteData(object):
         # 重建索引
         result.reset_index(drop=True, inplace=True)
 
+        # 复权处理
+        self.execute_rehabilitation(result, fq, fq_vol, inplace=True, skip_sort=True)
+
         return result
 
     def get_technical_analysis(self, security, indications, period='day', fq='pre', fq_vol=False, fields=None,
@@ -710,7 +922,7 @@ class QuoteData(object):
         """
 
         # 计算时需要预加载的数量
-        refer_num = 500
+        refer_num = self._options.get('refer_num', 0)
         start_date = formatter_st(start_date) if start_date else None
         end_date = formatter_st(end_date, self.get_quote_date())
         if not start_date and not count:
@@ -724,73 +936,26 @@ class QuoteData(object):
         elif count:
             prep_count = count + refer_num
 
-        indications = formatter_list(indications)
         cols_quote = ['open', 'close', 'high', 'low', 'volume', 'money', 'paused']
         fields = formatter_list(fields, cols_quote)
         df = self.get_quote_kdata(security, period=period, fq=fq, fq_vol=True, fields=cols_quote
                                   , start_date=prep_start, end_date=end_date, count=prep_count)
-        for v in indications:
-            v = [v] if type(v) is str else list(v)
-            c = v.pop(0)
-            func = globals().get(c)
-            if not func:
-                print('%s is not exists.' % c)
-                continue
-            if not hasattr(func, '__wrapped__'):
-                print('%s is unavailable.' % c)
-                continue
-            f_code = getattr(func, '__wrapped__').__code__
-            f_args = [x for x in f_code.co_varnames[:f_code.co_argcount]]
-            if len(f_args) > 0 and f_args[0] == 'X' and len(v) > 0 and type(v[0]) is not str:
-                f_args.pop(0)
-            kw = dict(axis=1)
-            for i in range(len(v)):
-                if i >= len(f_args):
-                    break
-                kw[f_args[i]] = v[i]
 
-            #获取返回值字段名
-            cols_fn = arithmetic_returns(f_code)
-            if not cols_fn:
-                cols_fn = c
-            if type(cols_fn) is str:
-                cols_fn = [cols_fn]
-            if not cols_fn:
-                raise Exception('columns name not support.')
-
-            # 字段重名处理
-            if [x for x in cols_fn if x in df.columns]:
-                c_arr = [str(x) for x in v]
-                if len(cols_fn) > 1:
-                    c_arr.insert(0, c)
-                    cols_fn = ['_'.join(c_arr) + '_' + x for x in cols_fn]
-                elif len(cols_fn) == 1:
-                    if cols_fn[0] == c:
-                        c_arr.insert(0, c)
-                        cols_fn = ['_'.join(c_arr)]
-                    else:
-                        c_arr.insert(0, cols_fn[0])
-                        cols_fn = ['_'.join(c_arr)]
-
-            # 提交指标函数处理
-            arithmetic_reset()
-            se = df.apply(func, **kw)
-            df[cols_fn] = pandas.DataFrame(se.tolist(), index=df.index)
+        self.execute_technical_analysis(df, indications, inplace=True, skip_sort=True)
 
         # 跳过停牌
         if skip_paused:
-            df = df[~df['paused']]
+            df = df[df['paused'] == 0]
 
         # 结果集数量显示
-        result = pandas.DataFrame()
+        result = pd.DataFrame()
         if not count:
-            dt_start = numpy.datetime64(
+            dt_start = np.datetime64(
                 datetime.datetime.fromtimestamp(time.mktime(start_date))) if start_date else None
-            dt_end = numpy.datetime64(datetime.datetime.fromtimestamp(time.mktime(end_date)))
+            dt_end = np.datetime64(datetime.datetime.fromtimestamp(time.mktime(end_date)))
             result = result.append(df[(df['datetime'] >= dt_start) & (df['datetime'] <= dt_end)], ignore_index=True)
         else:
-            c_groupby = df.groupby(by='code')
-            for x, y in c_groupby:
+            for x, y in df.groupby(by='code'):
                 result = result.append(y.iloc[-count:], ignore_index=True)
         result.reset_index(drop=True, inplace=True)
 
