@@ -122,7 +122,7 @@ class ArithmeticContainer(object):
             self._operator = None
             self._result = None
         else:
-            print('new operator', security)
+            # trace('new operator', security)
             self._security = security
             self._result = None
             self._operator = ArithmeticOperator()
@@ -141,7 +141,7 @@ class ArithmeticContainer(object):
         if item == 'result':
             return self._result
 
-        if item.lower() in ['ref', 'abs', 'max', 'min', 'sum',
+        if item.lower() in ['ref', 'std', 'abs', 'max', 'min', 'sum',
                             'ma', 'ema', 'mema', 'sma', 'hhv', 'llv']:
             if self._operator:
                 return getattr(self._operator, item.lower())
@@ -156,7 +156,7 @@ class ArithmeticOperator(object):
     def _fetch_stat(self):
         if self._index <= len(self._stats):
             self._stats.append(ArithmeticStatistics())
-        #print('current stats [%d]' % (self._index))
+        # print('current stats [%d]' % (self._index))
         stat = self._stats[self._index]
         self._index += 1
         return stat
@@ -180,7 +180,7 @@ class ArithmeticOperator(object):
             return r
         else:
             stat = self._fetch_stat()
-            #print('ref', x, n)
+            # print('ref', x, n)
             return stat.ref(x, n)
 
     def abs(self, x):
@@ -195,7 +195,7 @@ class ArithmeticOperator(object):
     def __getattr__(self, item):
         if item in ['open', 'close', 'high', 'low', 'volume', 'money']:
             return self.ref(item)
-        elif item in ['sum', 'ma', 'ema', 'mema', 'sma', 'hhv', 'llv']:
+        elif item in ['std', 'sum', 'ma', 'ema', 'mema', 'sma', 'hhv', 'llv']:
             stat = self._fetch_stat()
             return getattr(stat, item)
 
@@ -225,11 +225,19 @@ class ArithmeticStatistics(object):
         self._result = self._data[-n - 1] if len(self._data) > n else self._data[0]
         return self._result
 
+    '''
     def max(self, x):
         self.hhv(x, 0)
 
     def min(self, x):
         self.llv(x, 0)
+    '''
+
+    def std(self, x, n):
+        x = self._val(x)
+        self._update(x, n)
+        self._result = float(numpy.std(self._data[-n:]))
+        return self._result
 
     def sum(self, x, n=0):
         x = self._val(x)
@@ -277,7 +285,7 @@ def arithmetic_wrapper(func):
             arith = ArithmeticContainer()
             arith.reset(data['code'])
             arith.update(data)
-            if data['paused']:
+            if data['paused'] == 1:
                 return arith.result
             if 'X' in kwargs:
                 kwargs['X'] = eval(kwargs['X'].upper())
@@ -290,7 +298,6 @@ def arithmetic_wrapper(func):
                     arith.result = (0 for i in range(len(returns)))
                 else:
                     arith.result = 0
-            #arith.result = func(*args[1:], **kwargs)
             return arith.result
         else:
             return func(*args, **kwargs)
@@ -316,7 +323,6 @@ def arithmetic_returns(func_code):
     return result
 
 
-ARITH = ArithmeticContainer()
 OPEN = ArithmeticQuoteValue('open')
 CLOSE = ArithmeticQuoteValue('close')
 HIGH = ArithmeticQuoteValue('high')
@@ -329,35 +335,22 @@ def IF(C, T, F):
     return float(T) if C else float(F)
 
 
-def REF(X=CLOSE, N=0):
-    """获取参考数据
-
-    输入:
-        X: 数值或行情类型 [OPEN， CLOSE, HIGH, LOW, VOL, MONEY]
-        N: 统计天数
-
-    输出:
-        前N周期的行情数值
-
-    """
-    return ARITH.REF(X, N)
-
-
 def ABS(X):
     """绝对值"""
-    return ARITH.ABS(X)
+    return ArithmeticContainer().abs(X)
 
 
 def MAX(*args):
     """最大值"""
-    return ARITH.MAX(*args)
+    return ArithmeticContainer().max(*args)
 
 
 def MIN(*args):
     """最小值"""
-    return ARITH.MIN(*args)
+    return ArithmeticContainer().min(*args)
 
 
+@arithmetic_wrapper
 def SUM(X=CLOSE, N=5):
     """汇总
 
@@ -369,7 +362,37 @@ def SUM(X=CLOSE, N=5):
         前N周期的汇总数值
 
     """
-    return ARITH.SUM(X, N)
+    return ArithmeticContainer().sum(X, N)
+
+
+@arithmetic_wrapper
+def STD(X=CLOSE, N=0):
+    """标准差
+
+    输入:
+        X: 数值或行情类型 [OPEN， CLOSE, HIGH, LOW, VOL, MONEY]
+        N: 统计天数
+
+    输出:
+        前N周期的标准差
+
+    """
+    return ArithmeticContainer().std(X, N)
+
+
+@arithmetic_wrapper
+def REF(X=CLOSE, N=0):
+    """获取参考数据
+
+    输入:
+        X: 数值或行情类型 [OPEN， CLOSE, HIGH, LOW, VOL, MONEY]
+        N: 统计天数
+
+    输出:
+        前N周期的行情数值
+
+    """
+    return ArithmeticContainer().ref(X, N)
 
 
 @arithmetic_wrapper
@@ -384,7 +407,7 @@ def HHV(X=HIGH, N=5):
         前N周期的最大值
 
     """
-    return ARITH.HHV(X, N)
+    return ArithmeticContainer().hhv(X, N)
 
 
 @arithmetic_wrapper
@@ -399,7 +422,7 @@ def LLV(X=LOW, N=5):
         前N周期的最小值
 
     """
-    return ARITH.LLV(X, N)
+    return ArithmeticContainer().llv(X, N)
 
 
 @arithmetic_wrapper
@@ -414,7 +437,7 @@ def MA(X=CLOSE, N=5):
         MA: 移动平均
 
     """
-    return ARITH.MA(X, N)
+    return ArithmeticContainer().ma(X, N)
 
 
 @arithmetic_wrapper
@@ -428,7 +451,8 @@ def EMA(X=CLOSE, N=30):
     输出：
         EMA：指数移动平均
     """
-    return ARITH.EMA(X, N)
+    return ArithmeticContainer().ema(X, N)
+
 
 @arithmetic_wrapper
 def MEMA(X=CLOSE, N=5):
@@ -441,7 +465,8 @@ def MEMA(X=CLOSE, N=5):
     输出：
         MEMA：指数移动平均
     """
-    return ARITH.MEMA(X, N)
+    return ArithmeticContainer().mema(X, N)
+
 
 @arithmetic_wrapper
 def SMA(X=CLOSE, N=7, M=1):
@@ -456,5 +481,4 @@ def SMA(X=CLOSE, N=7, M=1):
     输出：
         SMA: 简单移动平均
     """
-    return ARITH.SMA(X, N, M)
-
+    return ArithmeticContainer().sma(X, N, M)
